#include <iostream>
using std::cout;
#include <iomanip>
using std::endl;
#include <cmath>
using std::pow;
#include "dvegas.h"
#include "omnicomp.h"

extern "C" { void trapfpe(); }  // guarantee abort on floating point exception

// define test integrands ---------------------------------------

// helper:
const double normalizedPower(const double& x, int exponent = 4)
{
  return pow(x, exponent) * (exponent+1);
}

void crudeMC(const double x[], const int k[], const double& wgt, const double aux[], double f[])
{
  f[0] = normalizedPower(aux[0]);
}

void adaptiveMC(const double x[], const int k[], const double& wgt, const double aux[], double f[])
{
  f[0] = normalizedPower(x[0]);
}

// --------------------------------------------------------------

int main(int argc, char* argv[])
{
  using HepSource::Dvegas;
  using HepSource::VEGAS;
  using HepSource::OmniComp;
  trapfpe();

  OmniComp::registerIntegrand(&crudeMC, "crudeMC");
  OmniComp::registerIntegrand(&adaptiveMC, "adaptiveMC");
  OmniComp omniComp(argc, argv);

  // correct results are 1.0 unless specified otherwise

  cout << endl << "+++++ no adaptation" << endl << endl;

  Dvegas dv1(1, 1, &crudeMC);
  dv1.attachOmniComp(&omniComp);
  VEGAS(dv1, 10000000, 2);

  cout << endl << "+++++ with adaptation" << endl << endl;

  Dvegas dv2(1, 50, 1, &adaptiveMC);
  dv2.attachOmniComp(&omniComp);
  VEGAS(dv2, 10000000, 3);
  
  return 0;
}
